--------------------------------------------------------------------------------------------------/
-- Company: Microsemi SoC Products Group
--
-- File: count24.vhd
-- File history:
--      1.0: July 22, 2016: Written
--      2.0: Nov. 28, 2018: UP_DN control signal added
--      <Revision number>: <Date>: <Comments>
--
-- Description: 
--
-- 24 bit counter
--
-- Targeted device: <Family::SmartFusion2> <Die::M2S050T_ES> <Package::896 FBGA>
-- Authors: Tim McCarthy, Jens Huettemann
--
--------------------------------------------------------------------------------------------------/ 
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity count24 is
port(
     CLK: in std_logic;
     UP_DN: in std_logic;
     RESETn: in std_logic;
     Q: out  std_logic_vector(23 downto 0)
     );
end count24;

architecture RTL of count24 is

-- signal declarations
signal Q_int: std_logic_vector(23 downto 0);
-- add Synplify directive syn_keep and attribute alspreserve to prevent Q[23:19] from being
-- removed in synthesis or layout.  Without these attributes the counter is pruned because the
-- counter output bits [23:19] don't drive any logic.  These will be driven off chip using Live Probe.
-- Explicit type declaration
attribute syn_keep : boolean;
attribute syn_keep of Q_int : signal is true;
attribute alspreserve : boolean;
attribute alspreserve of Q_int : signal is true;

begin
-- free running binary counter
CNT: process (CLK, RESETn)
begin
  if (RESETn = '0') then 
     Q_int <= (others => '0');
  elsif rising_edge(CLK) then
     if (UP_DN = '1') then
        Q_int <= Q_int + 1; -- counting up
     elsif (UP_DN = '0') then
        Q_int <= Q_int - 1; -- counting down
     else
        Q_int <= Q_int;
     end if;
  end if;
end process CNT;

Q <= Q_int;

end RTL;

